{ Screen for entering a three-digit code that activates something }
unit CodeScn;

interface

uses
  UIScrns, InputDrv, World;

type
  { TCodeScreen - Screen to enter a code }
  TCodeScreen = class(TUIScreen)
  private
    // The code to check for
    Code: string;
    // The target entity and command for successful code
    TargetEntity, TargetCommand: string;
    // The code so far
    EnteredCode: string[3];
    // Cursor position
    Cursor: Integer;
    // Attempt to use the entered code
    procedure AttemptCode;
  protected
    // TScreen methods
    procedure Draw; override;
  public
    // TInputHandler methods
    function KeyDown(Key: TInputKey): Boolean; override;
  public
    // TScreen methods
    function Pause: Boolean; override;
  public
    constructor Create;
    // Show the code screen for the given code, if the correct code is entered
    // then send the given command to the entity with the given name
    procedure ShowForCode(ACode, ATargetEntity, ATargetCommand: string);
  end;

implementation

uses
  Fonts, Engine, GfxDrv;

{ TCodeScreen }
constructor TCodeScreen.Create;
begin
  inherited Create;
end;

procedure TCodeScreen.AttemptCode;
var
  Target: TEntity;
begin
  // Check the code
  if EnteredCode=Code then begin
    // Find the target entity and send the command
    Target:=GGame.World.FindEntity(TargetEntity);
    if Assigned(Target) then Target.PerformCommand(TargetCommand);
  end;
  // Exit from the code screen
  GScreenManager.Screen:=nil;
end;

procedure TCodeScreen.Draw;
var
  W, H: Integer;
begin
  GRenderer.TextSize(Copy('000', 1, Cursor - 1), TBitFont(GResourceManager.DefaultFont), W, H);
  GRenderer.ShadeRectangle(0, BaseY + 50, HRes, BaseY + 104);
  GRenderer.DrawRectangle(-1, BaseY + 50, HRes + 1, BaseY + 104, 15);
  GRenderer.DrawText(BaseX + 126, BaseY + 60, 'Enter Code:', TBitFont(GResourceManager.DefaultFont), 15, 0);
  GRenderer.DrawRectangle(BaseX + 151, BaseY + 75, BaseX + 151 + 25, BaseY + 82 + 13, 0);
  GRenderer.DrawRectangle(BaseX + 150, BaseY + 74, BaseX + 150 + 25, BaseY + 81 + 13, 15);
  GRenderer.DrawText(BaseX + 153, BaseY + 78, EnteredCode, TBitFont(GResourceManager.DefaultFont), 24, 0);
  if ((GSystem.Milliseconds div 128) and 1) <> 0 then
    GRenderer.DrawText(BaseX + 153 + W, BaseY + 78, EnteredCode[Cursor], TBitFont(GResourceManager.DefaultFont), 5, 0);
  inherited Draw;
end;

function TCodeScreen.KeyDown(Key: TInputKey): Boolean;
begin
  if inherited KeyDown(Key) then Exit(True);
  case Key of
    ikUp, ikWheelDown: begin
      if EnteredCode[Cursor] in ['0'..'8'] then
        Inc(EnteredCode[Cursor])
      else
        EnteredCode[Cursor]:='0';
    end;
    ikDown, ikWheelUp: begin
      if EnteredCode[Cursor] in ['1'..'9'] then
        Dec(EnteredCode[Cursor])
      else
        EnteredCode[Cursor]:='9';
    end;
    ikLeft, ikRightButton: if Cursor > 1 then Dec(Cursor) else Cursor:=3;
    ikRight: if Cursor < 3 then Inc(Cursor) else Cursor:=1;
    ikLeftButton: if Cursor < 3 then Inc(Cursor) else AttemptCode;
    ik0..ik9: begin
      EnteredCode[Cursor]:=Char(Ord('0') + (Ord(Key) - Ord(ik0)));
      if Cursor < 3 then Inc(Cursor);
      Exit(True);
    end;
    ikEnter, ikMiddleButton, ikF: begin
      AttemptCode;
      Exit(True);
    end;
    ikBackspace: begin
      if Cursor in [2..3] then begin
        if not ((Cursor=3) and (EnteredCode[Cursor] <> '0')) then Dec(Cursor);
        EnteredCode[Cursor]:='0';
      end;
      Exit(True);
    end;
    ikEscape: begin
      GScreenManager.Screen:=nil;
      Exit(True);
    end;
  end;
  Result:=False;
end;

function TCodeScreen.Pause: Boolean;
begin
  Result:=False;
end;
                  
procedure TCodeScreen.ShowForCode(ACode, ATargetEntity, ATargetCommand: string);
begin
  // Store the values
  Code:=ACode;
  TargetEntity:=ATargetEntity;
  TargetCommand:=ATargetCommand;
  // Reset the entered code
  EnteredCode:='000';
  Cursor:=1;
  // Activate this screen
  GScreenManager.Screen:=Self;
end;

end.

