{ Pickup items }
unit EPickup;

interface

uses
  World;

type
  { TPickupItem - Represents an entity in the world that can be picked up }
  TPickupItem = class(TStaticMeshEntity)
  private
    // Property storage
    FSpeechText: string;
  protected
    // TEntity methods
    procedure Update; override;
  public
    // TEntity methods
    function PerformCommand(ACommand: string): Boolean; override;
  public
    constructor Create; override;
    // Called to pick up the item
    procedure PickUp;
  published
    // Speech text to appear when picking up the item
    property SpeechText: string read FSpeechText write FSpeechText;
  end;

implementation

uses
  Maths, Serial, Engine, SpchScn, EPetra, Sounds;

{ TPickupItem }          
constructor TPickupItem.Create;
begin
  inherited Create;
  Solid:=False;
  LightingType:=smltConstant;
end;

procedure TPickupItem.Update;
var
  T: TTransform;
begin
  inherited Update;
  T:=Transform;
  T.Rotation.Y:=ExtMod(T.Rotation.Y + 0.1, PI*2);
  Transform:=T;
end;

function TPickupItem.PerformCommand(ACommand: string): Boolean;
begin
  if inherited PerformCommand(ACommand) then Exit(True);
  if ACommand='pickup' then begin
    PickUp;
    Exit(True);
  end;
  Result:=False;
end;

procedure TPickupItem.PickUp;
begin
  TPetra(GGame.Player).AddToInventory(Name);
  World.DeleteEntityLater(Self);
  if SpeechText <> '' then GGame.SpeechScreen.Speech(sstPetra, SpeechText, '', '');
  GSound.PlayClip(TSoundClip(GResourceManager.GetResource('pickup.wav')), 32);
end;

initialization
  RegisterSerializableClass(TPickupItem);
end.
