unit ERotor;

interface

uses
  Maths, World;

type
  { TRotorState - Current rotor state }
  TRotorState = (rsStopped, rsRotating);

  { TRotor - Shows a rotating mesh }
  TRotor = class(TStaticMeshEntity)
  private
    // Property storage
    FState: TRotorState;
    FXRotationDelta: Single;
    FYRotationDelta: Single;
    FZRotationDelta: Single;
  protected
    // TEntity methods
    procedure Update; override;
  public
    // TEntity methods
    function PerformCommand(ACommand: string): Boolean; override;
  public
    constructor Create; override;
  published
    // Current rotor state
    property State: TRotorState read FState write FState;
    // Amount of rotation to add to RotationX every update
    property XRotationDelta: Single read FXRotationDelta write FXRotationDelta;
    // Amount of rotation to add to RotationY every update
    property YRotationDelta: Single read FYRotationDelta write FYRotationDelta;
    // Amount of rotation to add to RotationZ every update
    property ZRotationDelta: Single read FZRotationDelta write FZRotationDelta;
  end;

implementation

uses
  Serial;

{ TRotor }

procedure TRotor.Update;
var
  T: TTransform;
begin
  inherited Update;
  if State=rsRotating then begin
    T:=Transform;
    T.Rotation.X:=ExtMod(T.Rotation.X + XRotationDelta, PI*2);
    T.Rotation.Y:=ExtMod(T.Rotation.Y + YRotationDelta, PI*2);
    T.Rotation.Z:=ExtMod(T.Rotation.Z + ZRotationDelta, PI*2);
    Transform:=T;
  end;
end;

function TRotor.PerformCommand(ACommand: string): Boolean;
begin
  if inherited PerformCommand(ACommand) then Exit(True);
  if ACommand='stop' then begin
    State:=rsStopped;
    Exit(True);
  end;
  if ACommand='start' then begin
    State:=rsRotating;
    Exit(True);
  end;
  Result:=False;
end;

constructor TRotor.Create;
begin
  inherited Create;
  FState:=rsRotating;
  XRotationDelta:=0;
  YRotationDelta:=0.05;
  ZRotationDelta:=0;
end;

initialization
  RegisterSerializableClass(TRotor);
end.
