{ Note reader screen }
unit ReadrScn;

interface

uses
  UIScrns, Sounds;

type
  { TNoteReaderScreen - Screen for the disk note reader }
  TNoteReaderScreen = class(TUIScreen)
  private
    // List of note disks in Petra's possession
    NoteDisks: TUIList;
    // Currently selected disk's contents
    NoteText: TUIList;
    // Update the text for the selected disk
    procedure UpdateDiskText;
  protected
    // TScreen methods
    procedure Entered; override;
    procedure Exited; override;
  protected
    // TUIScreen methods
    procedure PerformCommand(Sender: TUIControl; ACommand: string); override;
    procedure Cancel; override;
  public
    // TScreen methods
    function Pause: Boolean; override;
  public
    constructor Create;
  end;

implementation

uses
  Engine, GfxDrv, Game, EPetra;

{ TNoteReaderScreen }
constructor TNoteReaderScreen.Create;
begin
  inherited Create;
  // Setup background style
  BackgroundStyle:=uisbShade;
  // Setup UI
  AddControl(TUILabel.Create(BaseX + 10, BaseY + 10, 'Available note disks:'));
  NoteDisks:=TUIList.Create(BaseX + 10, BaseY + 30, 300, 70, [], '');
  AddControl(TUILabel.Create(BaseX + 10, BaseY + 110, 'Selected disk''s contents:'));
  NoteText:=TUIList.Create(BaseX + 10, BaseY + 130, 300, 60, [], '');
  AddControl(NoteDisks);
  AddControl(NoteText);
end;

procedure TNoteReaderScreen.UpdateDiskText;
var
  Data, Line: string;
  I: Integer;
begin
  // Clear the current text
  NoteText.Clear;
  // Stop here if there isn't any disk selected
  if NoteDisks.ItemIndex=-1 then Exit;
  // Get the data for the disk
  Data:=GGame.World.GetData(NoteDisks.Items[NoteDisks.ItemIndex]);
  // Split the data into lines and add them to the text list
  Line:='';
  for I:=1 to Length(Data) do
    if Data[I]='_' then begin
      NoteText.AddItem(Line);
      Line:='';
    end else Line += Data[I];
  if Line <> '' then NoteText.AddItem(Line);
end;

procedure TNoteReaderScreen.Entered;
var
  I: Integer;
  Disks, Item: string;
begin
  inherited Entered;
  // Get the disks in the world
  Disks:=GGame.World.DataTable^.GetValue('disks');
  // Fill the disk list from Petra's inventory
  NoteDisks.Clear;
  for I:=0 to TPetra(GGame.Player).InventoryItemCount - 1 do begin
    Item:=TPetra(GGame.Player).InventoryItems[I];
    // Make sure this is a valid disk
    if Pos('$' + Item + '$', Disks) > 0 then
      NoteDisks.AddItem(Item);
  end;
  // Focus the disk list
  FocusedControl:=NoteDisks;
  // Update disk text
  UpdateDiskText;
  // Play computer on sound
  GSound.PlayClip(TSoundClip(GResourceManager.GetResource('disk.wav')));
end;

procedure TNoteReaderScreen.Exited;
begin
  // Play computer off sound
  GSound.PlayClip(TSoundClip(GResourceManager.GetResource('diskout.wav')));
  inherited Exited;
end;

procedure TNoteReaderScreen.PerformCommand(Sender: TUIControl; ACommand: string);
begin
  // A disk was selected
  if (Sender=NoteDisks) and (ACommand='_selected') then begin
    // Play disk read sound
    GSound.PlayClip(TSoundClip(GResourceManager.GetResource('diskread.wav')));
    UpdateDiskText;
  end;
end;

procedure TNoteReaderScreen.Cancel;
begin
  // Exit from the note reader screen
  GScreenManager.Screen:=nil;
end;

function TNoteReaderScreen.Pause: Boolean;
begin
  Result:=False;
end;

end.
